/**********************************************************************
 *                   GNU General Public License                       *
 * This file is a part of HIM.                                        *
 *                                                                    *
 * HIM is free software; you can redistribute it and/or modify it and *
 * are expected to follow the terms of the GNU General Public License *
 * as published by the Free Software Foundation; either version 2 of  *
 * the License, or (at your option) any later version.                *
 *                                                                    *
 * HIM is distributed in the hope that it will be useful, but WITHOUT *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY *
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public   *
 * License for more details.                                          *
 *                                                                    *
 * For the full text of the GNU General Public License,               *
 * write to: Free Software Foundation, Inc.,                          *
 *           675 Mass Ave, Cambridge, MA 02139, USA.                  *
 * or see:   http://www.gnu.org/licenses/gpl.html                     *
 **********************************************************************/

/********+*********+*********+*********+*********+*********+*********+*
 *                                                                    *
 *  By Robert Hallberg, November 1998 - May 2002                      *
 *                                                                    *
 *    This program contains the subroutines that calculate the        *
 *  surface wind stresses and fluxes of buoyancy or temperature and   *
 *  fresh water.  These subroutines will be called every time step,   *
 *  even if the wind stresses or buoyancy fluxes are constant in time *
 *  - in that case these routines return quickly without doing        *
 *  anything.  In addition, any I/O of forcing fields is controlled   *
 *  by set_forcing_output, located in this file.                      *
 *                                                                    *
 *    set_forcing is a small entry subroutine for the subroutines in  *
 *  this file.  It provides the external access to these subroutines. *
 *                                                                    *
 *    wind_forcing determines the wind stresses and places them into  *
 *  taux[][] and tauy[][].  Often wind_forcing must be tailored for   *
 *  a particular application - either by specifying file and variable *
 *  names or by providing appropriate internal expressions for the    *
 *  stresses.                                                         *
 *                                                                    *
 *    buoyancy_forcing determines the surface fluxes of buoyancy,     *
 *  temperature, and fresh water, as is appropriate.  A restoring     *
 *  boundary condition is implemented, but the code for any other     *
 *  boundary condition will usually be modified - either to specify   *
 *  file and variable names and which time level to read, or to set   *
 *  an internal expression for the variables.                         *
 *                                                                    *
 *  Variables written all in capital letters are defined in init.h    *
 *                                                                    *
 *     A small fragment of the grid is shown below:                   *
 *                                                                    *
 *    j+1  x ^ x ^ x   At x:  q                                       *
 *    j+1  > o > o >   At ^:  v, tauy                                 *
 *    j    x ^ x ^ x   At >:  u, taux                                 *
 *    j    > o > o >   At o:  h, fluxes.                              *
 *    j-1  x ^ x ^ x                                                  *
 *        i-1  i  i+1  At x & ^:                                      *
 *           i  i+1    At > & o:                                      *
 *                                                                    *
 *  The boundaries always run through q grid points (x).              *
 *                                                                    *
 ********+*********+*********+*********+*********+*********+*********+*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <init.h>
#include <HIM_io.h>
#include <HIM.h>
#include <metrics.h>


extern double hmask[NYMEM][NXMEM];   /* _mask are 0 for land points   */
extern double umask[NYMEM][NXMEM];   /* and 1 for ocean points.       */
extern double vmask[NYMEM][NXMEM];

extern double geolath[NYMEM][NXMEM]; /* The geographic latitude at h, */
extern double geolatq[NYMEM][NXMEM]; /* q, u, or v points in degrees  */
extern double geolatu[NYMEM][NXMEM]; /* of latitude or m.             */
extern double geolatv[NYMEM][NXMEM];
extern double geolonh[NYMEM][NXMEM]; /* The geographic longitude at h,*/
extern double geolonq[NYMEM][NXMEM]; /* q, u, or v points in degrees  */
extern double geolonu[NYMEM][NXMEM]; /* of longitude or m.            */
extern double geolonv[NYMEM][NXMEM];

extern struct params HIM_params;   /* HIM_params is a structure that  */
                                   /* contains a number of parameters */
                                   /* of the simulation which can be  */
                                   /* modified from a namelist file at*/
                                   /* run time.                       */
#ifdef RESTOREBUOY
# ifdef TEMPERATURE
static double T_Restore[NYMEM][NXMEM];/* The temperature toward which */
                                     /* the surface values are being  */
                                     /* restored, in C.               */
static double S_Restore[NYMEM][NXMEM];/* The salinity toward which    */
                                     /* the surface values are being  */
                                     /* restored, in psu.             */
# else
static double Dens_Restore[NYMEM][NXMEM];/* The potential density     */
                                     /* (coordinate variable) toward  */
                                     /* which the surface values are  */
                                     /* to be restored, in m s-2.     */
# endif
#endif
extern char directory2[];            /* The directory to use to save  */
                                     /* output files.                 */

extern int pe_here;                  /* The current processor's label.*/
extern int nx, ny;                   /* The largest index value in the*/
                                     /* x- and y- directions of the   */
                                     /* local computational domain.   */


static void set_forcing_output(struct forcing fluxes, time_type day_start);
static void wind_forcing(struct surface state, struct forcing *fluxes, time_type day);
static void buoyancy_forcing(struct surface state, struct forcing *fluxes,
                      time_type day, double dt);

void set_forcing(struct surface state, struct forcing *fluxes,
                 time_type day_start, time_type day_interval) {
/* This subroutine calls any of the other subroutines in this file    */
/* that are needed to specify the current surface forcing fields.     */

/* Arguments: state - A structure containing fields that describe the */
/*                    surface state of the ocean.                     */
/*  (out)     fluxes - A structure containing pointers to any possible*/
/*                     forcing fields.  Unused fields have NULL ptrs. */
/*  (in)      day_start - Start time of the fluxes.                   */
/*  (in)      day_interval - Length of time over which these fluxes   */
/*                           will be applied.                         */

  static int first_call = 1;
  double dt;            /* The length of time in seconds over which   */
                        /* the fluxes will be applied.                */
  time_type day_center; /* The central time of the fluxes.            */

  if (first_call) {
/* Set the various elements of fluxes to NULL - i.e. not used.        */
    int m;
    fluxes->taux = NULL; fluxes->tauy = NULL; fluxes->ustar = NULL;
    fluxes->buoy = NULL; fluxes->Net_heating = NULL; fluxes->PminusE = NULL;
    fluxes->sw = NULL; fluxes->lw = NULL; fluxes->latent = NULL;
    fluxes->sens = NULL; fluxes->lw_lat_sens = NULL; fluxes->evap = NULL;
    fluxes->precip = NULL; fluxes->liq_precip = NULL; fluxes->froz_precip = NULL;
    for (m=0;m<MAXTR;m++) {
      fluxes->tr_flux[m] = NULL; fluxes->tr_flux_names[m][0] = '\0';
    }
  }

  day_center = t_plus(day_start,t_div_int(day_interval,2));

  dt = time_to_double(day_interval,1);

#ifndef VARIABLE_WINDS
  if (first_call)
#endif
    wind_forcing(state, fluxes, day_center);

#ifndef ADIABATIC
# ifndef VARIABLE_BUOYFORCE
  if (first_call)
# endif
    buoyancy_forcing(state, fluxes, day_center, dt);
#endif

  if (first_call) {
    first_call = 0;
    set_forcing_output(*fluxes, day_start);
  }

}

static void wind_forcing(struct surface state, struct forcing *fluxes,
                         time_type day) {
/* This subroutine sets the surface wind stresses.                    */

/* Arguments: state - A structure containing fields that describe the */
/*                    surface state of the ocean.                     */
/*  (out)     fluxes - A structure containing pointers to any possible*/
/*                     forcing fields.  Unused fields have NULL ptrs. */
/*  (in)      day - Time of the fluxes.                               */

  static double taux[NYMEM][NXMEM];  /* taux & tauy are zonal & mer-  */
  static double tauy[NYMEM][NXMEM];  /* idional surface wind stress,  */
                                     /* in units of Pa.               */
#ifdef BULKMIXEDLAYER
  static double ustar[NYMEM][NXMEM]; /* The friction velocity, m s-1. */
#endif
  int i, j;

/* USER: Note that the following line must be edited to determine     */
/* whether the wind stress is internally generated or read from a     */
/* file.                                                              */
  int from_file = 0;          /* 0 if winds are internally generated, */
                              /* 1 if winds are read from a file.     */
  fluxes->taux = &taux;
  fluxes->tauy = &tauy;
#ifdef BULKMIXEDLAYER
  fluxes->ustar = &ustar;
#endif

  if (from_file) {
    int err, cdfid;
    int time_lev = 0;         /* With fields from a file, this must   */
                              /* be reset, depending on the time.     */
    char filename[200];       /* The names of the input file.         */

    strcpy(filename,HIM_params.inputdir);
    strcat(filename,"/MESO_str_2.cdf");

    err = open_input_file(filename,&cdfid,NULL);
    if (err != 0) {printf("Unable to open %s.\n",filename); quit(-10); }

#ifdef VARIABLE_WINDS
    printf("It is necessary to set time_lev or the file name in wind_forcing\n"
        "in surface_forcing.c to implement the VARIABLE_WINDS option.\n"
        "Replace this error message with the appropriate code.\n"
        "USER MESSAGE 1.\n"); quit(-1);
#endif

    Read_Field(cdfid,"UFLX", 'h', 1, time_lev, 1, &taux[0][0]);
    Read_Field(cdfid,"VFLX", 'h', 1, time_lev, 1, &tauy[0][0]);

    close_file(&cdfid);

/* Multiply by 0.1 to convert stresses from dyne/cm^2 to Pa.          */
    for (j=Y1;j<=ny;j++) for (i=X1;i<=nx;i++) {
      taux[j][i] *= 0.1;  tauy[j][i] *= 0.1;
    }

  } else {
#ifdef VARIABLE_WINDS
/*    This subroutine calculates time-varying wind stress fields      */
/*  and places them into taux[][] and tauy[][];  Of the two arguments */
/*  only day (the time of the wind stresses) is typically used, but   */
/*  it is conceivable that it would be desirable to have the wind     */
/*  stress depend on the surface velocities at time level lu.         */

# error "The wind stresses need to be defined with VARIABLE_WINDS."
/*  Calculate the wind stresses here.                                 */

#else
/*   Set the steady surface wind stresses, in units of Pa.            */
    for (j=Y1;j<=ny;j++) for (i=X1;i<=nx;i++) {
  /*  taux[j][i] = 0.0;  */
      taux[j][i] = 0.1 * (1.0 - cos(2.0*M_PI*(geolatu[j][i]-(LOWLAT))/(LENLAT)));
      tauy[j][i] = 0.0;
    }
#endif
  }

#ifdef BULKMIXEDLAYER
/*   Set the friction velocities here.                                */
/*   It may be desirable to set ustar to be a constant based on the   */
/* variability of the winds, rather than the mean stresses.  In this  */
/* case comment out the following subroutine call and replace the     */
/* defining expression for ustar.                                     */
/*   A zero mean wind offset of 0.02 N m-2 is added here. This value  */
/* is taken from the caption of figure 11.28 in Gill (1982).          */
  pass_var_2d(taux, 1, 1, 1, 0.0, 1, 0, TO_EAST,U_PT+VECTOR);
  pass_var_2d(tauy, 1, 1, 1, 0.0, 0, 0, TO_NORTH,V_PT+VECTOR);
  for (j=Y1;j<=ny;j++) for (i=X1;i<=nx;i++) {
    ustar[j][i] = sqrt(sqrt(0.5*(tauy[j-1][i]*tauy[j-1][i] +
        tauy[j][i]*tauy[j][i] + taux[j][i-1]*taux[j][i-1] +
        taux[j][i]*taux[j][i]))/RHO_0 + (0.02/RHO_0));
  }
#endif

}

static void buoyancy_forcing(struct surface state, struct forcing *fluxes,
                             time_type day, double dt) {
/*    This subroutine specifies the current surface fluxes of buoyancy*/
/*  temperature and fresh water.  It may also be modified to add      */
/*  surface fluxes of user provided tracers.                          */

/* Arguments: state - A structure containing fields that describe the */
/*                    surface state of the ocean.                     */
/*  (out)     fluxes - A structure containing pointers to any possible*/
/*                     forcing fields.  Unused fields have NULL ptrs. */
/*  (in)      day - Time of the fluxes.                               */
/*  (in)      dt - The amount of time over which the fluxes apply.    */

# ifdef TEMPERATURE                              /* start TEMPERATURE */
  static double PminusE[NYMEM][NXMEM]; /* Net fresh water flux, m s-1.*/
  static double Tforce[NYMEM][NXMEM];  /* Heat forcing divided by the */
                           /* density and heat capacity, in K m s-1.  */
# else
  static double buoy[NYMEM][NXMEM];    /* Buoyancy forcing in m2 s-3. */
# endif                                            /* end TEMPERATURE */

  int i, j, err, cdfid;
  char filename[200];                  /* The name of an input file.  */
  static int first_call = 1;

/* USER: Note that the following line must be edited to determine     */
/* whether the wind stress is internally generated or read from a     */
/* file.                                                              */
  int from_file = 0;       /* 0 if buoyancy forcing is internally     */
                           /* generated, 1 if it is read from a file. */


  if (first_call) {
#ifdef TEMPERATURE
    fluxes->PminusE = &PminusE;
    fluxes->Net_heating = &Tforce;
#else
    fluxes->buoy = &buoy;
#endif
  }

#ifdef RESTOREBUOY
  if ((from_file) && (first_call)) {
/* The next few lines set the file name - later string arguments      */
/* set the names in the NetCDF file of the fields to be read.         */
    strcpy(filename,HIM_params.inputdir);
    strcat(filename,"/MESO_SfcTS_2.nc");

    err = open_input_file(filename,&cdfid,NULL);
    if (err != 0) {printf("Unable to open %s.\n",filename); quit(-10); }

# ifdef TEMPERATURE
    Read_Field(cdfid,"Temp", 'h', 1, 0, 1, &T_Restore[0][0]);
    Read_Field(cdfid,"Salt", 'h', 1, 0, 1, &S_Restore[0][0]);
# else
    Read_Field(cdfid,"Buoy", 'h', 1, 0, 1, &Buoy_Restore[0][0]);
# endif

    close_file(&cdfid);

  } else if (first_call) {
      /* Set the surface values of T & S toward which to restore.     */
    printf("It is necessary to write the code to internally generate the values\n"
           "toward which the model is restored. Comment out this error message \n"
           "when the appropriate code exists in surface_forcing.c.\n"
           "USER MESSAGE 2.\n"); quit(-1);
# ifdef TEMPERATURE
    /*  The following is a plausible example.                         */
/*
    for (i=X1;i<=nx;i++) for (j=Y1;j<=ny;j++) {
      T_Restore[j][i] = 15.0 - 14.0 * cos(M_PI*(geolath[j][i]-(LOWLAT))/(LENLAT));
      S_Restore[j][i] = 35.0 - 0.5 * cos(2.0*M_PI*(geolath[j][i]-(LOWLAT))/(LENLAT));
    }
*/
# endif
  }

  for (j=Y1;j<=ny;j++) {
    for (i=X1;i<=nx;i++) {
      if (hmask[j][i] > 0) {
# ifdef TEMPERATURE
        Tforce[j][i] = (T_Restore[j][i] - (*(state.SST))[j][i]) * FLUXCONST;
        PminusE[j][i] = -(S_Restore[j][i] - (*(state.SSS))[j][i]) * FLUXCONST /
                         (*(state.SSS))[j][i];
# else                                             /* not TEMPERATURE */
        buoy[j][i] = (Dens_Restore[j][i] - (*(state.Rml))[j][i]) *
                     (G_EARTH*FLUXCONST/RHO_0);
# endif
      } else {
# ifdef TEMPERATURE
        Tforce[j][i] = 0.0;
        PminusE[j][i] = 0.0;
# else                                             /* not TEMPERATURE */
        buoy[j][i] = 0.0;
# endif
      }
    }
  }

#else                                              /* not RESTOREBUOY */

# ifdef VARIABLE_BUOYFORCE
/* delete the next "#error" line and replace it with appropriate code.*/
#  error "The fluxes need to be defined with VARIABLE_BUOYFORCE."
# else
  if (from_file) {

    printf("It is necessary to write the code to read the forcing fields from a file.\n"
           "Comment out this error message when the appropriate code exists\n"
           "in surface_forcing.c.  USER MESSAGE 3.\n"); quit(-1);

    strcpy(filename,HIM_params.inputdir);
    strcat(filename,"/MESO_SfcTS_2.nc");

/* The next few lines set the file name - later string arguments      */
/* set the names in the NetCDF file of the fields to be read.         */
    err = open_input_file(filename,&cdfid,NULL);
    if (err != 0) {printf("Unable to open %s.\n",filename); quit(-10); }
# ifdef TEMPERATURE
    Read_Field(cdfid,"Net_Heat", 'h', 1, 0, 1, &Tforce[0][0]);
    Read_Field(cdfid,"PminusE", 'h', 1, 0, 1, &PminusE[0][0]);
# else
    Read_Field(cdfid,"Buoy", 'h', 1, 0, 1, &buoy[0][0]);
# endif
    close_file(&cdfid);
  } else {
/*   This is where a steady buoyancy forcing would be set with an     */
/* internal function.                                                 */
    for (j=Y1;j<=ny;j++) for (i=X1;i<=nx;i++) {
/*    // This sets constant buoyancy forcing within a radius of       //
      // 200 km around a point.  The units of buoy are m2 s-3.        //
      double coslat, dist2;
      extern double g[NZ];// The interface reduced gravities in m s-2.//

      coslat = (DXh(j,i)*NXTOT*LENLAT)/(DYh(j,i)*NYTOT*LENLON);
      dist2 = (RE*RE*M_PI*M_PI/(180.0*180.0)) *
          (coslat * coslat * (geolonh[i][i] - 12.5) * (geolonh[i][i] - 12.5) +
               (geolath[j][i] - 40.0) * (geolath[j][i] - 40.0));
      if (dist2 <= (2.0e5*2.0e5)) buoy[j][i] = -1.528e-7*g[1];
      else buoy[j][i] = 0.0;
*/

#  ifdef TEMPERATURE
      Tforce[j][i] = 0.0;
      PminusE[j][i] = 0.0;
#  else
      buoy[j][i] = 0.0;
#  endif
    }
# endif                                     /* end VARIABLE_BUOYFORCE */
  }
#endif                                             /* end RESTOREBUOY */


  first_call = 0;
}

void average_forcing(struct forcing fluxes, double dt) {
/*   This subroutine offers forcing fields for time averaging.  These */
/* fields must first be registered in set_forcing_output (below).     */
/* This subroutine will typically not be modified, except when new    */
/* forcing fields are added.                                          */

/* Arguments: fluxes - A structure containing pointers to any possible*/
/*                     forcing fields.  Unused fields have NULL ptrs. */
/*  (in)      dt - The amount of time over which to average.          */

  static int av_ref_taux = -1, av_ref_tauy = -1;
  static int av_ref_PminusE = -1, av_ref_LwLatSens= -1;

  if (av_ref_taux < 0) {
    av_ref_taux = get_field_ref(fluxes.taux);
    av_ref_tauy = get_field_ref(fluxes.tauy);
    av_ref_PminusE = get_field_ref(fluxes.PminusE);
    av_ref_LwLatSens = get_field_ref(fluxes.lw_lat_sens);
  }

  if (av_ref_taux > 0) average_field(fluxes.taux, av_ref_taux);
  if (av_ref_tauy > 0) average_field(fluxes.tauy, av_ref_tauy);
  if (av_ref_PminusE > 0) average_field(fluxes.PminusE, av_ref_PminusE);
  if (av_ref_LwLatSens > 0)
    average_field(fluxes.lw_lat_sens, av_ref_LwLatSens);
}

static void set_forcing_output(struct forcing fluxes, time_type day_start) {
/*   This subroutine prepares for any output of averages of any       */
/* temporally varying forcing fields and writes out a file containing */
/* steady fields, but only if that file does not already exist.       */

/* Arguments: fluxes - A structure containing pointers to any possible*/
/*                     forcing fields.  Unused fields have NULL ptrs. */
/*  (in)      day_start - Time at which the fluxes first apply.       */

  char filename[40];     /* The file name of the save file.           */
  char filepath[120];    /* The path (dir/file) to the file.          */
  int cdfid, timeid;
  size_t err;
  struct varcdfinfo varinfo[10];
  vardesc vars[10];
  int no_vars = 0;
  int taux_id = -1, tauy_id = -1, buoy_id = -1;

  sprintf(filename,"Steady_forcing.%d.%d",NXTOT,NYTOT);
  strcpy(filepath, directory2);
  strcat(filepath, filename);
  /* If the file filepath exists, the following call will return 1.   */
  err = open_input_file(filename,&cdfid,NULL);
  if (err != 0) {
    /* The steady forcing file does not exist, so it can be created.  */
    /* Add blocks like the following to add fields to this file.      */
# ifndef VARIABLE_WINDS
    if ((fluxes.taux != NULL) && (fluxes.tauy != NULL)) {
      vardesc taux_desc = {"taux","Zonal Wind Stress",'u','1','1',"Pascal", 'd'};
      vardesc tauy_desc = {"tauy","Zonal Wind Stress",'v','1','1',"Pascal", 'd'};
      taux_id = no_vars; tauy_id = no_vars+1; no_vars += 2;
      vars[taux_id] = taux_desc;
      vars[tauy_id] = tauy_desc;
    }
# endif
# ifndef VARIABLE_BUOYFORCE
    if (fluxes.buoy != NULL) {
      vardesc buoy_desc = {"buoy","Buoyancy forcing",'h','1','1',"meter2 second-3", 'd'};
      buoy_id = no_vars; no_vars++;
      vars[buoy_id] = buoy_desc;
    }
# endif
    if (no_vars>0) {
      create_file(filepath, vars, no_vars, &cdfid, &timeid, varinfo);
      err = 1;
      if (taux_id >= 0)
        err *= write_field(cdfid, vars[taux_id], varinfo[taux_id], 0, (*(fluxes.taux))[0]);
      if (tauy_id >= 0)
        err *= write_field(cdfid, vars[tauy_id], varinfo[tauy_id], 0, (*(fluxes.tauy))[0]);
      if (buoy_id >= 0)
        err *= write_field(cdfid, vars[buoy_id], varinfo[buoy_id], 0, (*(fluxes.buoy))[0]);
      if (err == 0)
       printf("Problems saving steady forcing fields.\n");
    }
  }
  close_file(&cdfid);

  /*   At this point, time averaged forcing fields are set up.        */
  /* Comment or add lines to change the fields that will be output.   */
  {
    specify_saves_file("avforce", directory2, 'a', day_start, HIM_params.daysave,
         HIM_params.daymax, HIM_params.saveint, HIM_params.saves_per_file);
# ifdef VARIABLE_WINDS
    {
      vardesc taux_desc = {"taux","Zonal Wind Stress",'u','1','a',"Pascal", 'f'};
      vardesc tauy_desc = {"tauy","Zonal Wind Stress",'v','1','a',"Pascal", 'f'};
      if (fluxes.taux != NULL) set_up_save_field(fluxes.taux,NULL,taux_desc);
      if (fluxes.tauy != NULL) set_up_save_field(fluxes.tauy,NULL,tauy_desc);
    }
# endif
# ifdef VARIABLE_BUOYFORCE
    {
      vardesc buoy_desc = {"buoy","Buoyancy forcing",'h','1','a',"meter2 second-3", 'f'};
      vardesc LwLatSens_desc = {"LwLatSens","Combined longwave, latent, "
          "and sensible heating",'h','1','a',"Watt meter-2", 'f'};
      vardesc PminusE_desc = {"PmE","Precipitation minus Evaporation",
          'h','1','a',"meter second-1", 'f'};
      if (fluxes.buoy != NULL) set_up_save_field(fluxes.buoy,NULL,buoy_desc);
      if (fluxes.PminusE != NULL)
        set_up_save_field(fluxes.PminusE,NULL,PminusE_desc);
      if (fluxes.lw_lat_sens != NULL)
        set_up_save_field(fluxes.lw_lat_sens,NULL,LwLatSens_desc);
    }
# endif

  }
}

void register_forcing_restarts(void) {
/* This subroutine is used to register any fields in surface_forcing.c*/
/* that should be written to or read from the restart file.           */

  /* This subroutine is deliberately empty. */

}
